const argv = require("minimist")(process.argv.slice(2));
const simpleGitPromise = argv.g
  ? require("simple-git/promise")(argv.g)
  : require("simple-git/promise")();
const pageToCapture = argv.p || "./index.html";
const puppeteer = require("puppeteer");
const fs = require("fs");
const GIFEncoder = require("gifencoder");
const pngFileStream = require("png-file-stream");
const tmp = "./tmp";

const getLog = async () => {
  const logObj = await simpleGitPromise.log();
  return logObj.all;
};

const deleteFolderRecursive = path => {
  var files = [];
  if (fs.existsSync(path)) {
    files = fs.readdirSync(path);
    files.forEach(function(file, index) {
      var curPath = path + "/" + file;
      if (fs.lstatSync(curPath).isDirectory()) {
        // recurse
        deleteFolderRecursive(curPath);
      } else {
        // delete file
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
  }
};

const run = async () => {
  const browser = await puppeteer.launch();
  const page = await browser.newPage();

  await page.setViewport({
    width: 1280,
    height: 800
  });

  const logs = await getLog();

  if (!fs.existsSync(tmp)) {
    fs.mkdirSync(tmp);
  }

  for (let [i, log] of logs.entries()) {
    await simpleGitPromise.checkout(log.hash);
    if (fs.existsSync(argv.p)) {
      await page.goto(argv.p);
      await page.screenshot({
        path: `tmp/capture${logs.length - i}.png`,
        fullPage: true
      });
    }
  }

  await simpleGitPromise.checkout("master");

  await browser.close();

  const encoder = new GIFEncoder(1280, 800);

  const stream = pngFileStream("tmp/capture??.png")
    .pipe(
      encoder.createWriteStream({
        repeat: -1,
        delay: 500,
        quality: 10
      })
    )
    .pipe(fs.createWriteStream("timelapse.gif"));

  stream.on("finish", function() {
    deleteFolderRecursive(tmp);
  });
};

// run();

(async () => {
  const status = await simpleGitPromise.status();
  console.log(status);
})();
